/** CLIENT-SIDE 
 * Anurag Bhandari (anurag.bhd@gmail.com)
 * 29-01-2009
 * 
 * DESCRIPTION:
 * This program scans the shared directory for files in it, extracts metainfo 
 * for each file in the file list, and dumps the extracted metainfo about each 
 * file into the MySQL database.
 * 
 * CLASSES USED:
 * - DirectoryScanner
 * - ExtractMetaInfo
 * 
 * 
 */

package p2p;

import java.io.*;
import java.util.*;

public class PostToFile {
    
    /* INSTANCE VARIABLES */
    private String fileName, filePath, fileMD5, fileType;
    private String[] sharedFolders;
    private long fileSize;
    private Date lastModified;
    private String artist, album, song_title;
    private String[] metaInfo;
    
    /* CONSTRUCTOR */
    public PostToFile(String[] folders) {
        sharedFolders = folders;
    }
    
    /* METHODS */
    /**
     * Checks if the type/extension of a file is one of the allowed types
     * 
     * @param extensionList a comma separated list of allowed file extensions, for example - mp3,wma,mpg,avi
     * @param fileExtension the file extension to be found in the list of allowed extensions
     * @return  true if the file extension found in the list, false otherwise
     */
    public static boolean filter(String extensionList, String fileExtension) {
        boolean flag = false;
        String[] allowedTypes = extensionList.split(",");
        for(String type : allowedTypes) {
            if(type.equals(fileExtension)) {
                flag = true;
            }
        }
        return(flag);
    }
    
    /**
     * This our main method which actually generates a text file with the name "file-list.txt" 
     * with information about each file in a directory (initialized by the constructor) as contents.
     * <p>
     * Entry for each file in "file-list.txt" is separated by the newline symbol.
     * <br>
     * Individual file attributes information are separated by "*".
     * 
     * @throws java.lang.Exception
     */
    public void generateFile() throws Exception {
        
        DirectoryScanner d = new DirectoryScanner();
        File[] listOfFiles;
        BufferedWriter out = new BufferedWriter(new FileWriter("file-list.txt"));
        
        for(int i=0; i<sharedFolders.length; i++) {
            /* Dump files-list into a file */
            
            // Scan the directory containing files to be shared
            listOfFiles = d.filePathList(sharedFolders[i]);
            
            
            // Do this for each file in the list of files in scanned directory
            for(File file : listOfFiles) {
                // Extract the metainfo out of the current file
                ExtractMetaInfo f = new ExtractMetaInfo(file.getPath());
                fileName = f.name();
                filePath = f.path();
                fileSize = f.size();
                fileType = f.type();
                lastModified = f.lastModifiedDate();
                
                // Proceed only if the file type is one of specified in the 
                // filter list. The allowed file types should be mentioned 
                // in the first argument space, separated by commas.
                if(filter("mp3,wma,wav,avi,mpg,mpeg,wmv,mov,mp4,divx", fileType)) {
                    if(!file.isDirectory()) {
                        // Calculate the md5sum only if it's not a directory, 
                        // otherwise, exception will occur telling something like  
                        // md5 cannot be applied to directory
                        fileMD5 = f.md5sum();
                        
                        // Metadata of file
                        metaInfo = f.metaData(fileType);
                        artist = metaInfo[0];
                        album = metaInfo[1];
                        song_title = metaInfo[2];
                        
                        // Writing data to file    
                        out.write(fileName+'*'+filePath+'*'+fileSize+'*'+fileType+'*'+lastModified+'*'+fileMD5+'*'+song_title+'*'+artist+'*'+album);
                        out.newLine();

                    }
                }
            }
        }
        out.close();
    }
}