/** CLIENT-SIDE 
 * Anurag Bhandari (anurag.bhd@gmail.com)
 * Gagandeep Singh Walia (gagan.jld@hmail.com)
 * 28-03-09
 * 
 * DESCRIPTION:
 * The main GUI interface that the end user actually sees. Contains all the features 
 * that make up for the peer app.
 * 
 */

package p2pclient;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.table.DefaultTableModel;
import java.util.Map;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import javax.swing.JOptionPane;
import javax.swing.JProgressBar;
import javax.swing.JDialog;
import javax.swing.UIManager;
import java.awt.Dimension;
import java.awt.Toolkit;
import p2p.*;

public class ClientUI extends javax.swing.JFrame {

    Socket client;
    InputStream instream;
    OutputStream outstream;
    ObjectOutputStream obout;
    ObjectInputStream obin;
    PrintWriter outwriter;
    static List<File> files;
    static List<String> filesname;
    JDialog od;
    
    
    /**
     * Converts any object into corresponding bytes array. This is done 
     * for the purpose of sending data efficiently over the socket.
     * 
     * @param obj any type of object to covert into bytes
     * @return converted bytes array of the input object
     * @throws java.io.IOException
     */
    public static byte[] toBytes(Object obj) throws java.io.IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(bos);
        oos.writeObject(obj);
        oos.flush();
        oos.close();
        bos.close();
        byte[] data = bos.toByteArray();
        return data;
    }
    
    /**
     * Converts a byte array (received over the socket) into its corresponding 
     * object form (which can then later be cast into its appropriate form, say a String).
     * 
     * @param bytes the input byte array received over the socket
     * @return the corresponding object
     * @throws java.io.IOException
     * @throws java.lang.ClassNotFoundException
     */
    public static Object toObject(byte[] bytes) throws IOException, ClassNotFoundException {
        Object data = null;
        ByteArrayInputStream bos = new ByteArrayInputStream(bytes);
        ObjectInputStream ois = new ObjectInputStream(bos);
        data = ois.readObject();
        ois.close();
        bos.close();
        return data;
    }

    /** Creates new form ClientUI */
    public ClientUI() {
        initComponents();
        jButton5.setEnabled(false);
        od = new OptionsDialog();
        // Making the window centered
        // Idea from: http://www.java-forums.org/awt-swing/3491-jframe-center-screen.html
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        int w = getSize().width;
        int h = getSize().height;
        int x = (dim.width-w)/2;
        int y = (dim.height-h)/2;
        setLocation(x, y);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane2 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        jTextField2 = new javax.swing.JTextField();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jProgressBar1 = new javax.swing.JProgressBar();
        jLabel4 = new javax.swing.JLabel();
        jToolBar1 = new javax.swing.JToolBar();
        jButton7 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        jButton1 = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        jButton6 = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JToolBar.Separator();
        jTextField1 = new javax.swing.JTextField();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenuItem4 = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("ShareNet - Client");

        jTable1.setAutoCreateRowSorter(true);
        jTable1.setFont(jTable1.getFont().deriveFont(jTable1.getFont().getSize()+1f));
        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "NAME", "MD5", "ARTIST", "ALBUM", "SIZE"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jTable1.setRowHeight(20);
        /* Making the column "MD5" of "jTable1" invisible */
        /* Idea: http://www.experts-exchange.com/Programming/Languages/Java/Q_20394392.html */
        int iColumn = 1;
        // set column width
        jTable1.getColumnModel().getColumn(iColumn).setMaxWidth(0);
        jTable1.getColumnModel().getColumn(iColumn).setMinWidth(0);
        // set column header width (to avoid the ... sign)
        jTable1.getTableHeader().getColumnModel().getColumn(iColumn).setMaxWidth(0);
        jTable1.getTableHeader().getColumnModel().getColumn(iColumn).setMinWidth(0);

        iColumn = 4;
        // set column width
        jTable1.getColumnModel().getColumn(iColumn).setMaxWidth(0);
        jTable1.getColumnModel().getColumn(iColumn).setMinWidth(0);
        // set column header width (to avoid the ... sign)
        jTable1.getTableHeader().getColumnModel().getColumn(iColumn).setMaxWidth(0);
        jTable1.getTableHeader().getColumnModel().getColumn(iColumn).setMinWidth(0);
        jScrollPane2.setViewportView(jTable1);
        jTable1.getColumnModel().getColumn(0).setResizable(false);
        jTable1.getColumnModel().getColumn(0).setPreferredWidth(300);

        jTextField2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField2ActionPerformed(evt);
            }
        });

        jButton4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/download.png"))); // NOI18N
        jButton4.setText("Download Now!");
        jButton4.setMaximumSize(new java.awt.Dimension(175, 24));
        jButton4.setMinimumSize(new java.awt.Dimension(175, 24));
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        jButton5.setText("Search");
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });

        jLabel1.setFont(jLabel1.getFont().deriveFont(jLabel1.getFont().getStyle() | java.awt.Font.BOLD, jLabel1.getFont().getSize()+4));
        jLabel1.setText("Welcome to ShareNet");

        jLabel2.setFont(new java.awt.Font("Tahoma", 1, 11));
        jLabel2.setText("Search Results");

        jLabel4.setText("Download Progress");

        jToolBar1.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);
        jToolBar1.setToolTipText("Commonly used actions");

        jButton7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/login.png"))); // NOI18N
        jButton7.setToolTipText("Sign In");
        jButton7.setFocusable(false);
        jButton7.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton7.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton7ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton7);

        jButton3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/options.png"))); // NOI18N
        jButton3.setToolTipText("Options");
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton3);
        jToolBar1.add(jSeparator2);

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/download-toolbar.png"))); // NOI18N
        jButton1.setToolTipText("Download the selected file");
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton1);
        jToolBar1.add(jSeparator1);

        jButton6.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/exit.png"))); // NOI18N
        jButton6.setToolTipText("Quit ShareNet");
        jButton6.setFocusable(false);
        jButton6.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton6.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jButton6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton6ActionPerformed(evt);
            }
        });
        jToolBar1.add(jButton6);
        jToolBar1.add(jSeparator4);

        jTextField1.setText("Enter server address to override settings in Options > Server");
        jTextField1.setToolTipText("Enter server address to override settings in Options > Server");
        jTextField1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField1ActionPerformed(evt);
            }
        });
        jToolBar1.add(jTextField1);

        jMenu1.setText("File");

        jMenuItem2.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/login-small.png"))); // NOI18N
        jMenuItem2.setText("Sign In");
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem2);

        jMenuItem4.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/download-small.png"))); // NOI18N
        jMenuItem4.setText("Download");
        jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem4ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem4);
        jMenu1.add(jSeparator3);

        jMenuItem1.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/exit-small.png"))); // NOI18N
        jMenuItem1.setText("Quit");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem1);

        jMenuBar1.add(jMenu1);

        jMenu2.setText("Edit");

        jMenuItem3.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        jMenuItem3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/options-small.png"))); // NOI18N
        jMenuItem3.setText("Options");
        jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem3ActionPerformed(evt);
            }
        });
        jMenu2.add(jMenuItem3);

        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 707, Short.MAX_VALUE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 707, Short.MAX_VALUE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 371, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 348, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jProgressBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 347, Short.MAX_VALUE)
                            .addComponent(jLabel4))))
                .addContainerGap())
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 731, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTextField2, javax.swing.GroupLayout.DEFAULT_SIZE, 522, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton5, javax.swing.GroupLayout.PREFERRED_SIZE, 179, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 228, Short.MAX_VALUE)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(jProgressBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jButton4, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    /**
     * Performs all the operations required to log the client out of the server.
     */
    public void logOut() {
        try {
            outstream.write((byte) 4);
            outstream.flush();
            client.close();
            outstream.close();
            instream.close();
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        jTextField1.setEditable(true);
        jButton5.setEnabled(false);
        jMenuItem2.setText("Sign In");
        jMenuItem2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/login-small.png")));
        jButton7.setToolTipText("Sign In");
        jButton7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/login.png")));
    }
    
    /**
     * Logs the client into the server if not currently logged in and vice-versa. It also 
     * updates the UI accordingly.
     */
    public void toggleSignIn() {
        if (jMenuItem2.getText().equals("Sign In")) {
            String serverAdd = null;
            if(!jTextField1.getText().equals("") && !jTextField1.getText().equals("Enter server address to override settings in Options > Server")) {
                serverAdd = jTextField1.getText();
            }
            else {
                String[] serverAddress = null;
                try {
                    File serverAddFile = new File("server-address.txt");
                    ReadTextFile fileObj = new ReadTextFile();
                    //@SuppressWarnings("static-access")
                    serverAddress = fileObj.getContents(serverAddFile).split("\\|");
                } catch(Exception ex) {
                    System.out.println("It seems you have not selected any downloads folder yet.");
                }

                if(!serverAddress[0].equals("")) {
                    serverAdd = serverAddress[0];
                    jTextField1.setText(serverAdd);
                }
                else {
                    System.out.println("It seems you have not entered the server address yet. Go to Options > Server.");
                    JOptionPane.showMessageDialog(null, "It seems you have not entered the server address yet.\nGo to Options > Server.", "Error", JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
            BufferedInputStream bufstream = null;
            byte[] buffer = new byte[1024];
            try {
                client = new Socket(InetAddress.getByName(serverAdd), 4321);
                outstream = client.getOutputStream();
                instream = client.getInputStream();
                obout = new ObjectOutputStream(outstream);
                obin = new ObjectInputStream(instream);
                System.out.println("Connection to server established");
            } catch (IOException ex) {
                System.out.println("The server might be down right now. Try later.");
                JOptionPane.showMessageDialog(null, "The server might be down right now. Try later.", "Error", JOptionPane.ERROR_MESSAGE);
                return; // Do not execute the code in this function that follows

            }

            String fileToUpload = "file-list.txt";
            System.out.println("Sending file: " + fileToUpload);
            try {
                bufstream = new BufferedInputStream(new FileInputStream(fileToUpload));
            } catch (FileNotFoundException ex) {
                System.out.println("Please share a folder first and scan it to create the initial files list. Go to Options > Shared Folders.");
                JOptionPane.showMessageDialog(null, "Please share a folder first and scan it to create the initial files list.\nGo to Options > Shared Folders.", "Error", JOptionPane.WARNING_MESSAGE);
                return; // Do not execute the code in this function that follows

            }

            int num;
            try {
                while ((num = bufstream.read(buffer)) != -1) {
                    obout.writeInt(num);
                    obout.flush();
                    outstream.write(buffer, 0, num);
                    outstream.flush();
                }
                System.out.println("File list sent to the server");

            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            jTextField1.setEditable(false);
            jButton5.setEnabled(true);
            jMenuItem2.setText("Sign Out");
            jMenuItem2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/logout-small.png")));
            jButton7.setToolTipText("Sign Out");
            jButton7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/p2pclient/images/logout.png")));
            new FileTransferThread().start();
        } else if (jMenuItem2.getText().equals("Sign Out")) {
            logOut();
        }
    }

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        downloadFile();
    }//GEN-LAST:event_jButton4ActionPerformed

    private void downloadFile() {
        String myFileMD5, myFileName;
        float myFileSize;
        int selectedRow;
        selectedRow = jTable1.getSelectedRow();
        try {
            // The column argument is 1 because we have specified file MD5 in the second column; so, this can change in future
            myFileMD5 = jTable1.getValueAt(selectedRow, 1).toString();
            // The column argument is 0 because we have specified file name in the first column; so, this can change in future
            myFileName = jTable1.getValueAt(selectedRow, 0).toString();
            // The column argument is 0 because we have specified file size in the fifth column; so, this can change in future
            myFileSize = Float.parseFloat(jTable1.getValueAt(selectedRow, 4).toString());

            System.out.println("Downloading file: " + myFileName);
            try {
                outstream.write((byte) 2);
                outstream.flush();
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            byte bytearray[] = null;
            try {
                bytearray = toBytes(myFileMD5);
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            int size = bytearray.length;
            try {
                obout.writeInt(size);
                obout.flush();
                outstream.write(bytearray);
                outstream.flush();
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            bytearray = null;
            try {
                size = obin.readInt();
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            bytearray = new byte[size];
            try {
                instream.read(bytearray);
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            Map<InetAddress, String> result = null;
            try {
                try {
                    result = (Map<InetAddress, String>) toObject(bytearray);
                } catch (ClassNotFoundException ex) {
                    Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            // Idea: http://www.faqs.org/docs/javap/c12/s3.html
            Set entries = result.entrySet();
            Iterator entryIter = entries.iterator();
            System.out.println("Result has been recieved");
            if (entryIter.hasNext()) {
                System.out.println("Result is not empty");
                // Begin downloading by initiating the downloadThread
                new downloadThread(jProgressBar1, entryIter, myFileName, myFileSize).start();
            } else {
                System.out.println("No peer found");
                JOptionPane.showMessageDialog(null, "No peer found", "Error", JOptionPane.ERROR_MESSAGE);
            }
            jTextField2.setEditable(true);
        } catch (Exception e) {
            System.out.println("Please select a file to download.");
            JOptionPane.showMessageDialog(null, "Please select a file to download.", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
    /**
     * The method that performs actual searching. It sends the search keywork over the socket 
     * to the server app. The server app in turn performs the searching into the MySQL database 
     * using the received search keyword. It then sends the search result back to the client, 
     * again over the socket, and finally, this client-side function reads the received 
     * list of search results and returns it for further use.
     * 
     * @return the list of search results received from the server app
     * @throws java.io.IOException
     * @throws java.lang.ClassNotFoundException
     */
    private ArrayList<String> performSearch() throws IOException, ClassNotFoundException {
        try {
            outstream.write((byte) 3);
            outstream.flush();
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }

        // Send search keyword to server
        byte[] bytearray = toBytes(jTextField2.getText());
        int size = bytearray.length;
        try {
            obout.writeInt(size);
            obout.flush();
            outstream.write(bytearray);
            outstream.flush();
            System.out.println("Search keyword sent to server");
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }

        // Receive the result from server
        bytearray = null;
        try {
            size = obin.readInt();
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }

        bytearray = new byte[size];
        try {
            instream.read(bytearray);
            System.out.println("Search result received from server");
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        ArrayList<String> resultArrayList = (ArrayList<String>) toObject(bytearray);
        return (resultArrayList);
    }

    /**
     * Gets the search results returned by performSearch() and adds each one of them 
     * to the "Search Results" table in the UI.
     * 
     * @throws java.io.IOException
     * @throws java.lang.ClassNotFoundException
     */
    private void displaySearchResult() throws IOException, ClassNotFoundException {
        String[] row;
        DefaultTableModel model = (DefaultTableModel) jTable1.getModel();
        model.setNumRows(0);
        @SuppressWarnings("static-access")
        ArrayList<String> result = performSearch();
        for (int i = 0; i < result.size(); i++) {
            row = result.get(i).split(":");
            model.addRow(row);
        }
    }

private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
    try {
        try {
            displaySearchResult();
            } catch (ClassNotFoundException ex) {//GEN-LAST:event_jButton5ActionPerformed
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

private void jTextField2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextField2ActionPerformed
    try {
        try {
            displaySearchResult();
            } catch (ClassNotFoundException ex) {//GEN-LAST:event_jTextField2ActionPerformed
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        } catch (IOException ex) {
            Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

private void jTextField1ActionPerformed(java.awt.event.ActionEvent evt) {
    toggleSignIn();
}

private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {
    System.exit(0);
}

private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {
    toggleSignIn();
}

private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {
    od.setVisible(true);
}

private void jMenuItem4ActionPerformed(java.awt.event.ActionEvent evt) {
    downloadFile();
}

private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {
    toggleSignIn();
}

private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {
    od.setVisible(true);
}

private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {
    downloadFile();
}

private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {
    System.exit(0);
}




    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JToolBar.Separator jSeparator4;
    private javax.swing.JTable jTable1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JToolBar jToolBar1;
    // End of variables declaration//GEN-END:variables
}

class downloadThread extends Thread {

    JProgressBar progressBar;
    Iterator entryIter;
    Object key, value;
    InetAddress peerIP;
    String filePath, myFileName;
    float myFileSize;

    downloadThread(JProgressBar br, Iterator it, String fi, float xy) {
        progressBar = br;
        entryIter = it;
        myFileName = fi;
        myFileSize = xy;

        String[] downloadFolder = null;
        try {
            File downloadLocationFile = new File("download-location.txt");
            ReadTextFile fileObj = new ReadTextFile();
            //@SuppressWarnings("static-access")
            downloadFolder = fileObj.getContents(downloadLocationFile).split("\\|");
        } catch(Exception ex) {
        System.out.println("It seems you have not selected any downloads folder yet.");
        }

        if(!downloadFolder[0].equals("")) {
            // Is it Windows?
            if(UIManager.getSystemLookAndFeelClassName().equals("com.sun.java.swing.plaf.windows.WindowsLookAndFeel")) {
                 myFileName = downloadFolder[0] + "\\" + myFileName;
            }
            // Is it any other OS other than Windows - Linux, Mac, BSD?
            else {
                myFileName = downloadFolder[0] + "/" + myFileName;
            }
        }
    }

    @Override
    public void run() {
        boolean downloaded = false;
        byte bytearray[] = null;
        int size;
        while ((!downloaded) && (entryIter.hasNext())) {
            Map.Entry entry = (Map.Entry) entryIter.next();
            key = entry.getKey();
            value = entry.getValue();
            peerIP = (InetAddress) key;
            filePath = (String) value;
            File downfile = new File(myFileName);
            Socket clientconnection = null;
            try {
                clientconnection = new Socket(peerIP, 5555);
            } catch (IOException ex) {
                System.out.println("Peer(s) not available right now, or socket configuration is wrong. Please try again later.");
                JOptionPane.showMessageDialog(null, "Peer(s) not available right now, or socket configuration is wrong. Please try again later.", "Error", JOptionPane.ERROR_MESSAGE);
                break;
            }
            OutputStream outout = null;
            InputStream inin = null;
            BufferedOutputStream buffout = null;
            ObjectOutputStream oboutout;
            try {
                inin = clientconnection.getInputStream();
                outout = clientconnection.getOutputStream();
                oboutout = new ObjectOutputStream(outout);
                bytearray = ClientUI.toBytes(filePath);
                size = bytearray.length;
                oboutout.writeInt(size);
                oboutout.flush();
                outout.write(bytearray);
                outout.flush();
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);

            }
            byte[] buffer = new byte[1024];
            try {
                buffout = new BufferedOutputStream(new FileOutputStream(downfile));
            } catch (FileNotFoundException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }

            int num;
            float counter = (float) 0.0;
            progressBar.setValue(0);
            progressBar.setStringPainted(true);
            System.out.println("Downloading...");

            try {
                while ((num = inin.read(buffer)) != -1) {
                    buffout.write(buffer, 0, num);
                    counter = (float) (counter + (float) num);
                    int x = (int) (((float) counter / myFileSize) * (float) 100);
                    progressBar.setValue(x);
                }
                System.out.println("File downloaded");
                downloaded = true;
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }


            try {
                buffout.close();
                inin.close();
                outout.close();
            } catch (IOException ex) {
                Logger.getLogger(ClientUI.class.getName()).log(Level.SEVERE, null, ex);
            }

        }
    }
}