/** SERVER-SIDE 
 * Anurag Bhandari (anurag.bhd@gmail.com)
 * Gagandeep Singh Walia (gagan.jld@hmail.com)
 * 
 * DESCRIPTION:
 * This is the main thread that runs on the server app and listens on the port 4321 
 * for activity. It contains handlers which respond to different types of activities 
 * detected.
 * 
 */

package p2pserver;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.sql.ResultSet;
import java.io.File;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.FileNotFoundException;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;

class ClientThread extends Thread {

    static Map<Socket, List<String>> Table = new HashMap<Socket, List<String>>();
    Socket client;
    OutputStream outstream = null;
    InputStream instream = null;
    ObjectInputStream obin;
    ObjectOutputStream obout;
    private static String query_view_create,  query_view_delete,  query_search;
    private static String searchKey,  fileName,  fileMD5,  songArtist,  songAlbum, fileSize;
    private static ArrayList<String> resultArrayList;
    private static ArrayList<String> searchResult;
    private static String url,  username,  password;
    Connection connection;
    boolean ready = false;

    /**
     * Converts any object into corresponding bytes array. This is done 
     * for the purpose of sending data efficiently over the socket.
     * 
     * @param obj any type of object to covert into bytes
     * @return converted bytes array of the input object
     * @throws java.io.IOException
     */
    public static byte[] toBytes(Object obj) throws java.io.IOException{
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(bos);
        oos.writeObject(obj);
        oos.flush();
        oos.close();
        bos.close();
        byte [] data = bos.toByteArray();
        return data;
    }

    /**
     * Converts a byte array (received over the socket) into its corresponding 
     * object form (which can then later be cast into its appropriate form, say a String).
     * 
     * @param bytes the input byte array received over the socket
     * @return the corresponding object
     * @throws java.io.IOException
     * @throws java.lang.ClassNotFoundException
     */
    public static Object toObject(byte[] bytes) throws IOException, ClassNotFoundException {
        Object data = null;
        ByteArrayInputStream bos = new ByteArrayInputStream(bytes);
        ObjectInputStream ois = new ObjectInputStream(bos);
        data = ois.readObject();
        ois.close();
        bos.close();
        return data;
    }
    
    /* CONSTRUCTOR */
    /**
     * Initializes the socket and corresponding streams; fetches the files-list file 
     * from the connected client; posts the entries from the files-list file to the 
     * MySQL database.
     * 
     * @param skt the socket to initialize
     * @throws java.sql.SQLException
     * @throws java.lang.ClassNotFoundException
     */
    ClientThread(Socket skt) throws SQLException, ClassNotFoundException {
        client = skt;
        try {
            outstream = client.getOutputStream();
            instream = client.getInputStream();
            obout = new ObjectOutputStream(outstream);
            obin = new ObjectInputStream(instream);

        } catch (IOException ex) {
            Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        /* Getting the "file-list.txt" file from connected client */
        System.out.println(client.getRemoteSocketAddress());
        System.out.println("Starting download");
        String clientIP = client.getInetAddress().toString().split("/")[1];
        File downFile = new File(clientIP);
        BufferedOutputStream buffout = null;
        byte[] buffer = new byte[1024];
        try {
            buffout = new BufferedOutputStream(new FileOutputStream(downFile));
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
        }

        int num;
        int flag = 0;
        int kick;
        try {
            while (true) {
                if ((kick = obin.readInt()) == 1024) {
                    //System.out.println(kick);
                    num = instream.read(buffer);
                    buffout.write(buffer, 0, num);
                    buffout.flush();
                } else if (kick == -1) {
                    flag = 1;
                    break;
                } else {
                    //System.out.println(kick);
                    buffer = new byte[kick];
                    num = instream.read(buffer);
                    buffout.write(buffer, 0, kick);
                    buffout.flush();
                    flag = 1;
                    break;
                }
            }
            System.out.println("Files-list file downloaded");

        } catch (IOException ex) {
            Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
        }

        if (flag == 1) { // Execute the following only after its confirmed that the files list file has finished downloading.
            /* MySQL driver info */
            String driverName = "com.mysql.jdbc.Driver"; // MySQL JDBC driver

            Class.forName(driverName);


            /* MySQL server & DB info */
            // Getting this info from text file
            File configFile = new File("p2p_config.txt");
            ReadTextFile File1 = new ReadTextFile();
            @SuppressWarnings("static-access")
            String[] configValues = File1.getContents(configFile).split("\\|");

            String serverName = configValues[1];
            String dbName = configValues[3];
            url = "jdbc:mysql://" + serverName + "/" + dbName; // a JDBC url

            username = configValues[5];
            password = configValues[7];

            // Initializing the "connection" to null
            connection = null;

            // The actual connection to DB
            connection = DriverManager.getConnection(url, username, password);
            System.out.println("Connection: " + connection);

            Statement stmt = connection.createStatement();

            /* Reading the list for files for this client from its file and posting it to DB */
            File filesListFile = new File(clientIP);
            ReadTextFile File2 = new ReadTextFile();
            @SuppressWarnings("static-access")
            String[] fileEntries = File2.getContents(filesListFile).split("\\|");
            String[] entryAttribs;
            String filePath;
            String query_clean_files, query_clean_metainfo, query_files, query_metainfo; 
            query_clean_files = "DELETE FROM files WHERE peer_ip='" + clientIP + "'";
            stmt.executeUpdate(query_clean_files);
            query_clean_metainfo = "DELETE FROM metainfo WHERE peer_ip='" + clientIP + "'";
            stmt.executeUpdate(query_clean_metainfo);
            for (int x = 0; x < fileEntries.length; x++) {
                entryAttribs = fileEntries[x].split("\\*");
                // doing this for escaping back slashes in Windows paths; to make it MySQl-friendly
                filePath = replace(entryAttribs[1], "\\", "\\\\");
                // doing this for escaping apostrophies; to make it MySQl-friendly
                filePath = replace(filePath, "\'", "\\\'");
                query_files = "INSERT INTO files (peer_ip, file_name, file_path, " +
                        "file_size, file_type, file_last_modified, file_md5) " +
                        "values ('" + clientIP + "', '" + replace(entryAttribs[0], "\'", "\\\'") + "', '" + filePath + "', '" +
                        entryAttribs[2] + "', '" + entryAttribs[3] + "', '" + entryAttribs[4] +
                        "', '" + entryAttribs[5] + "')";
                stmt.executeUpdate(query_files);
                query_metainfo = "INSERT INTO metainfo (peer_ip, song_title, " +
                        "artist, album) " + "values ('" + clientIP + "', '" + replace(entryAttribs[6], "\'", "\\\'") +
                        "', '" + replace(entryAttribs[7], "\'", "\\\'") + "', '" +
                        replace(entryAttribs[8], "\'", "\\\'") + "')";
                stmt.executeUpdate(query_metainfo);
            }

            // Closing MySQL connection
            connection.close();

            /* If all the above code in this constructor executes, then it means the program is ready 
            *  to run the thread below. So, we set the instance variable "ready" to true.
            */
            ready = true;
        }
    }

    /**
     * Replaces all instances of a String in a String. I am using this method mostly to 
     * replace all apostrophies "'" with "\'" for making the complete string usable 
     * for the SQL syntax.
     * 
     * @param s string to perform replacement in; the original string
     * @param f string to replace
     * @param r string to replace with
     * @return the string with replacement performed
     */
    public static String replace(String s, String f, String r) {
        if (s == null) {
            return s;
        }
        if (f == null) {
            return s;
        }
        if (r == null) {
            r = "";
        }
        int index01 = s.indexOf(f);
        while (index01 != -1) {
            s = s.substring(0, index01) + r + s.substring(index01 + f.length());
            index01 += r.length();
            index01 = s.indexOf(f, index01);
        }
        return s;
    }

    @Override
    public void run() {
        int i;
        try {
            System.out.println("New user has logged in: " + client.getInetAddress());
            while ((i = (int) instream.read()) != -1) {
                // This first case is not being used anymore.
                if (i == 1) {
                    System.out.println("i reached in if");
                    byte[] bytearray = null;
                    int size = obin.readInt();
                    bytearray = new byte[size];
                    i = instream.read(bytearray);
                    System.out.println("files " + this + " is sharing are:");
                    System.out.println(this + ":added files information to server\n");
                }
                // Handler for returning a map of IP-to-filepath to the client corresponding to MD5 received from it
                else if (i == 2) {
                    byte[] bytearray = null;
                    int size = obin.readInt();
                    bytearray = new byte[size];
                    instream.read(bytearray);
                    try {
                        searchKey = (String) toObject(bytearray);
                    } catch (ClassNotFoundException ex) {
                        Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
                    }
                    try {

                        /* The actual connection to DB */
                        connection = DriverManager.getConnection(url, username, password);

                        /* Displaying connection related info */
                        System.out.println("Connection: " + connection);

                        /* Creating objects to handle queries */
                        Statement stmt = connection.createStatement();
                        ResultSet rs;

                        query_search = "SELECT * FROM files WHERE file_md5='" + searchKey + "'";
                        rs = stmt.executeQuery(query_search);

                        Map<InetAddress, String> result = new HashMap<InetAddress, String>();
                        String filePath;
                        String peerIPAddress;
                        while (rs.next()) {
                            filePath = rs.getString("file_path");
                            peerIPAddress = rs.getString("peer_ip");
                            result.put(InetAddress.getByName(peerIPAddress), filePath);
                        }
                        System.out.println("Server is sending result to " + client.getInetAddress());
                        bytearray = toBytes(result);
                        size = bytearray.length;
                        obout.writeInt(size);
                        obout.flush();
                        outstream.write(bytearray);
                        outstream.flush();
                        System.out.println("Result sent! The client should now know from whom to download the file.");
                        connection.close();
                    } catch (Exception e) {
                        System.out.println(e.getMessage());
                    }
                }
                // Handler for searching in the MySQL database and returning search results against the keyword received from client.
                else if (i == 3) {
                    try {
                        // Receive the search keyword
                        byte[] bytearray = null;
                        int size = obin.readInt();
                        bytearray = new byte[size];
                        instream.read(bytearray);
                        try {
                            searchKey = (String) toObject(bytearray);
                        } catch (ClassNotFoundException ex) {
                            Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
                        }
                        System.out.println("Search query received: " + searchKey);

                        // Perform search and send back result to client
                        searchResult = performSearch(searchKey);
                        bytearray = toBytes(searchResult);
                        size = bytearray.length;
                        obout.writeInt(size);
                        obout.flush();
                        outstream.write(bytearray);
                        outstream.flush();

                        System.out.println("Search result sent to client");
                    } catch (IOException ex) {
                        Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
                // Handler for Sign In / Sign Out button on client side
                else if (i == 4) {
                    System.out.println(client.getInetAddress() + " has logged out");
                    this.stop();
                }
            }
        } catch (IOException ex) {
            Logger.getLogger(ClientThread.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
   
    /* The method that performs actual searching */
    public ArrayList<String> performSearch(String key) {
        try {

            /* The actual connection to DB */
            connection = DriverManager.getConnection(url, username, password);

            /* Displaying connection related info */
            System.out.println("Connection: " + connection);

            /* Creating objects to handle queries */
            Statement stmt = connection.createStatement();
            ResultSet rs;


            query_view_create = "CREATE VIEW complete_file_info AS SELECT metainfo.file_id, metainfo.song_title, metainfo.artist, metainfo.album, files.file_name, files.file_path, files.file_size, files.file_type, files.file_last_modified, files.file_md5 FROM metainfo INNER JOIN files where metainfo.file_id=files.file_id";
            stmt.executeUpdate(query_view_create);

            query_search = "SELECT * FROM complete_file_info WHERE file_name like '%" + searchKey + "%' OR song_title like '%" + searchKey + "%' OR artist like '%" + searchKey + "%' OR album like '%" + searchKey + "%'";
            rs = stmt.executeQuery(query_search);

            resultArrayList = new ArrayList<String>();
            while (rs.next()) {
                fileName = rs.getString("file_name");
                fileMD5 = rs.getString("file_md5");
                songArtist = rs.getString("artist");
                songAlbum = rs.getString("album");
                fileSize = rs.getString("file_size");
                resultArrayList.add(fileName + ":" + fileMD5 + ":" + songArtist + ":" + songAlbum + ":" + fileSize);
            }

            query_view_delete = "DROP VIEW complete_file_info";
            stmt.executeUpdate(query_view_delete);

            /* Closing the MySQL connection */
            connection.close();


        } catch (Exception e) {
            // Could not find the database driver
            System.out.println(e.getMessage());
        }
        return (resultArrayList);
    }
}